/*
 * File: aspellbridge.cpp
 * Author: Federico Boschetti <federico.boschetti.73@gmail.com>
 *
 * Copyright © 2009 Perseus Project - Tufts University <http://www.perseus.tufts.edu>
 *
 * This file is part of OcroPerseus.
 *
 * OcroPerseus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * OcroPerseus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with OcroPerseus.  If not, see <http://www.gnu.org/licenses/>
 *
 */

/*
 * C++ interface to Aspell APIs, used by JNI Jaspeller
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <aspell.h>
#include "jaspellbridge_Jaspeller.h"

static char list[10000];
static const char * lang="*";
static AspellCanHaveError * ret;
static AspellSpeller * speller;
static AspellConfig * config;

/*
 * Catenates results in a space separated list string
 */
static const char * make_word_list(const char * inputStr, int have, const AspellWordList *wl){
    AspellStringEnumeration * els = aspell_word_list_elements(wl);
    const char * wd;
    list[0]='\0';
    strcat(list,inputStr);
    if(!have){
        while ( (wd = aspell_string_enumeration_next(els)) != 0) {
            strcat(list," ");
            strcat(list,wd);
        }
    }
    delete_aspell_string_enumeration(els);
    return list;
}

/*
 *  The Aspell Engine is static, so it keeps in memory information for a specific language, until the jLang parameter is changed (for ex., switching from Greek to English or vice versa. When jLang is set to "~", the speller is destroyed and memory is released.
 */
JNIEXPORT jstring JNICALL Java_jaspellbridge_Jaspeller_aspellQuery(JNIEnv * env, jclass jc, jstring jInputStr, jstring jLang){
    const char * inputStr=(*env).GetStringUTFChars(jInputStr,0);
    const char * cLang=(*env).GetStringUTFChars(jLang,0);
    if(!strcmp(cLang,"~")){
        delete_aspell_speller(speller);
        return (*env).NewStringUTF("del");
    }
    if(strcmp(cLang,lang)){
        config = new_aspell_config();
        aspell_config_replace(config, "lang", cLang);
        aspell_config_replace(config, "encoding", "utf-8");
        ret = new_aspell_speller(config);
        delete_aspell_config(config);
        speller = to_aspell_speller(ret);
        config = aspell_speller_config(speller);
        lang=cLang;
    }
    int have=aspell_speller_check(speller, inputStr, -1);
    const char * res=make_word_list(inputStr, have, aspell_speller_suggest(speller, inputStr, -1));
    return (*env).NewStringUTF(res);
}
